<?php
/**
 * MRWP_Orders_List_Table class
 *
 * @package mondialrelay
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

// WP_List_Table class inclusion
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class MRWP_History_List_Table extends WP_List_Table {

	/**
	 * The order modes for filtering
	 *
	 * @var array
	 */
	private $order_modes = array();

	/**
	 * The order dates for filtering
	 *
	 * @var array
	 */
	private $order_dates = array();

	/**
	 * The order status we look at
	 *
	 * @var array
	 */
	protected $watched_statuses = array();

	public function __construct() {
		parent::__construct(
			array(
				'singular' => 'expedition',     //singular name of the listed records
				'plural'   => 'expeditions',    //plural name of the listed records
				'ajax'     => true,             //ajax support
				'screen'   => 'mondial-relay_page_wp-mondialrelay-expeditions',
			)
		);

		// All WooCommerce status with custom ones and without cancelled, refunded, failed, draft (completed added for V2.7)
		$this->watched_statuses = array_diff(array_keys( wc_get_order_statuses() ), [ "wc-cancelled", "wc-refunded", "wc-failed", "wc-checkout-draft" ]);
	}

	/**
	 * @return bool
	 */
	public function ajax_user_can() {
		return current_user_can( 'shop_manager' );
	}

	public function prepare_items( $all_records = false ) {

		// column_headers
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array( $columns, $hidden, $sortable );

		/*
		 * Tri
		 *
		 * Problème :
		 * - Soit on utilise le tri de WooCommerce et on est limités à 'ID', 'name', 'type', 'rand', 'date' et 'modified'
		 * - Soit on n’utilise pas wc_get_orders()... on fait ce qu’on veut mais on doit tout refaire
		 */

		// @see https://github.com/woocommerce/woocommerce/wiki/wc_get_orders-and-WC_Order_Query
		$args = array(
			// Only get orders with an ExpeditionNum
			'meta_query'   => array( [
				'key'     => 'ExpeditionNum',
				'compare' => 'EXISTS',
			] ),
			'orderby'	   => ! empty( $_REQUEST['orderby'] ) && '' != $_REQUEST['orderby'] ? $_REQUEST['orderby'] : 'ID',
			'order'		   => ! empty( $_REQUEST['order'] ) && '' != $_REQUEST['order'] ? $_REQUEST['order'] : 'desc',
		);

		// If we have a valid status filter, use it, otherwise accept all statuses
		if ( isset( $_GET['filter_etat'] ) && in_array( $_GET['filter_etat'], $this->watched_statuses ) ) {
			$args['post_status'] = array( $_GET['filter_etat'] );
		} else {
			$args['post_status'] = $this->watched_statuses;
		}

		// If we have a valid mode filter, use it, otherwise accept all MR modes
		$default_modes = MRWP_modes::all_modes();
		if ( isset( $_GET['filter_mode'] ) && in_array( $_GET['filter_mode'], $default_modes ) ) {
			$args['meta_query'][] = array(
				'key'   => 'Mondial Relay Shipping Code',
				'value' => array( $_GET['filter_mode'] ),
			);
		} else {
			$args['meta_query'][] = array(
				'key'   => 'Mondial Relay Shipping Code',
				'value' => $default_modes,
			);
		}

		// If we have a date filter, add it to the query args
		if ( isset( $_GET['date_from_iso'] ) and strlen( $_GET['date_from_iso'] ) == 10 ) {
			if ( isset( $_GET['date_to_iso'] ) and strlen( $_GET['date_to_iso'] ) == 10  ) {
				$args['date_created'] = $_GET['date_from_iso'] . '...' . $_GET['date_to_iso'];
			} else {
				$today = new DateTimeImmutable();
				$args['date_created'] = $_GET['date_from_iso'] . '...' . $today->format( 'Y-m-d' );
			}
		}

		// $all_records is true for CSV export, false for paginated display
		if ( ! $all_records ) {
			// Orders per page — try and retrieve the WooCommerce setting
			$per_page = (int)get_user_meta( get_current_user_id(), 'edit_shop_order_per_page', true );
			if ( $per_page < 1 ) {
				$per_page = 10;
			}

			$args['limit']    = $per_page;
			$args['paged']    = $this->get_pagenum();
			$args['paginate'] = true;

			$all_orders = wc_get_orders( $args );
			$orders     = $all_orders->orders;

			$this->set_pagination_args(
				array(
					'total_items'	=> $all_orders->total,
					'per_page'	    => $per_page,
					'total_pages'	=> $all_orders->max_num_pages,
					'orderby'	    => ! empty( $_REQUEST['orderby'] ) && '' != $_REQUEST['orderby'] ? $_REQUEST['orderby'] : 'ID',
					'order'		    => ! empty( $_REQUEST['order'] ) && '' != $_REQUEST['order'] ? $_REQUEST['order'] : 'desc'
				)
			);
		} else {
			$args['limit']    = -1;
			$args['paginate'] = false;

			$all_orders = wc_get_orders( $args );
			$orders     = $all_orders;
		}

		$data = array();
		foreach ( $orders as $order ) {
			$data[] = array(
				'Num_Dossier'    => $order->get_id(),
				'Status'         => $order->get_status(),
				'Mode_Livraison' => $order->get_meta( 'Mondial Relay Shipping Code' ),
				'Livraison'      => $order->get_meta( 'ExpeditionStatus' ),
				'Date'           => $order->get_date_created()->format ('Y-m-d'),
				'Client'         => $order->get_formatted_shipping_full_name(),
				'Adresse'        => $order->get_formatted_shipping_address(),
				'Expedition'     => $order->get_meta( 'ExpeditionNum' ),
			);
		}

		// Save list of orders
		$this->items = $data;
	}

	/**
	 * This function is used by the display code
	 *
	 * @return int
	 */
	public function has_items() {
		return count( $this->items );
	}

	/**
	 * Returns the list of the table columns in order
	 *
	 * @return array
	 */
	public function get_columns() {
		$columns = array(
			'cb'             => '<input type="checkbox" />', //Render a checkbox instead of text
			'Num_Dossier'    => __( 'Order', 'woocommerce' ),
			'Expedition'     => __( 'Expédition', 'mondialrelay-wordpress'),
			'Client'         => __( 'Client', 'woocommerce' ),
			'Adresse'        => __( 'Ship to', 'woocommerce' ),
			'Date'           => __( 'Date', 'woocommerce' ),
			'Mode_Livraison' => __( 'Mode de Livraison', 'mondialrelay-wordpress'),
			'Status'         => __( 'Status', 'woocommerce' ),
			'Livraison'      => __( 'Livraison', 'mondialrelay-wordpress' ),
		);

		return $columns;
	}

	/**
	 * Returns the name of the default primary column.
	 *
	 * @return string Name of the default primary column
	 */
	protected function get_primary_column_name() {
		return 'Num_Dossier';
	}

	/**
	 * Returns the list of sortable columns.
	 *
	 * @return array list of sortable columns
	 */
	function get_sortable_columns() {
		$sortable_columns = array(
			'Num_Dossier'    => array( 'ID', 'desc' ),   // The second element sets the initial sorting order.
			'Date'           => array( 'date', 'asc' ),
			'Client'         => array( 'name', 'asc' ),
		);

		return $sortable_columns;
	}

	/**
	 * Display our stuff before the standard table.
	 */
	public function display() {

		wp_nonce_field( 'mrwp-list-nonce', 'mrwp_list_nonce' );
		$this->display_labels_buttons();

		parent::display();
	}

	/**
	 * Display the #labels-buttons block
	 */
	public function display_labels_buttons() {
		require_once( __DIR__ . '/parts/admin-history-buttons.php' );
	}

	/**
	 * Extra controls to be displayed between bulk actions and pagination.
	 *
	 * @since 3.1.0
	 *
	 * @param string $which
	 */
	protected function extra_tablenav( $which ) {
		?>
		<div class="alignleft actions">
		<?php
		if ( 'top' === $which ) {
			ob_start();

			$selected_status = ! empty( $_GET['filter_etat'] ) ? $_GET['filter_etat'] : '';

			?>
			<label for="filter-by-status" class="screen-reader-text"><?php _e( 'Filtrer par état', 'mondialrelay-wordpress' ); ?></label>
			<select name="filter_etat" id="filter-by-status">
				<option<?php selected( $selected_status, '' ); ?> value=""><?php _e( 'Tous les états', 'mondialrelay-wordpress' ); ?></option>
				<?php
				foreach ( $this->watched_statuses as $status ) {
					printf(
						"<option %s value='%s'>%s</option>\n",
						selected( $selected_status, $status, false ),
						esc_attr( $status ),
						esc_html( wc_get_order_status_name( $status ) )
					);
				}
				?>
			</select>

			<?php
			$shipping_modes = MRWP_modes::all_modes();
			$selected_mode  = ! empty( $_GET['filter_mode'] ) ? $_GET['filter_mode'] : '';
			?>
			<label for="filter-by-mode" class="screen-reader-text"><?php _e( 'Filtre par mode', 'mondialrelay-wordpress' ); ?></label>
			<select name="filter_mode" id="filter-by-mode">
				<option<?php selected( $selected_mode, '' ); ?> value=""><?php _e( 'Tous les modes', 'mondialrelay-wordpress' ); ?></option>
				<?php echo MRWP_modes::all_modes_as_options( $selected_mode );?>
			</select>

			<?php
			$date_from = ! empty( $_GET['date_from'] ) ? $_GET['date_from'] : '';
			$date_from = ! empty( $_GET['date_to'] ) ? $_GET['date_to'] : '';
			?>
			<style>
			.datepicker { width: 145px; }
			.column-Livraison .updated { color: green; }
			.column-Livraison .error { color: red; }
			</style>
			<label for="filter-date-from"><?php _e( 'À partir du&nbsp;:', 'mondialrelay-wordpress' ); ?></label>
			<input type="text" class="datepicker" name="date-from" id="filter-date-from" value="<?php echo esc_attr( $date_from ); ?>">
			<input type="hidden" name="date-from-iso" id="date-from-iso" value="<?php echo esc_attr( $date_from ); ?>">

			<label for="filter-date-to"><?php _e( 'jusqu’au&nbsp;:', 'mondialrelay-wordpress' ); ?></label>
			<input type="text" class="datepicker" name="date-to" id="filter-date-to" value="<?php echo esc_attr( $date_from ); ?>">
			<input type="hidden" name="date-to-iso" id="date-to-iso" value="<?php echo esc_attr( $date_from ); ?>">
			<?php

			$output = ob_get_clean();

			if ( ! empty( $output ) ) {
				echo $output;
				submit_button( __( 'Filter' ), '', 'filter_action', false, array( 'id' => 'filter-submit' ) );
				submit_button( __( 'Export' ), '', 'export_action', false, array( 'id' => 'export-submit' ) );
			}
		}
		?>
		</div>
		<?php
	}

	/**
	 * Handles the column output.
	 *
	 * @param object $item The current item.
	 * @param string $column_name The current column name.
	 */
	protected function column_default( $item, $column_name ) {

		// Escape values
		$order_id = intval( $item['Num_Dossier'] );

		switch( $column_name ) {
			case 'Num_Dossier':
				return $order_id;

			case 'Status':
				return esc_html( wc_get_order_status_name( $item['Status'] ) );

			case 'Mode_Livraison':
				return '<span class="mode-liv" data-id="' . $order_id . '">' . esc_html( $item['Mode_Livraison'] ) . '</span> ';

			case 'Livraison':
				return '<span class="statut-liv" data-expedition_num="' . esc_attr( $item['Expedition'] ) . '">' . esc_html( class_MRWP_helpers::etat_expedition( $item['Livraison'] ) ) . '</span> ';

			case 'Client':
			case 'Date':
			case 'Expedition':
				return esc_html( $item[ $column_name ] );

			case 'Adresse':
				return $item['Adresse'];

			default:
				return print_r( $item, true ); //Show the whole array for troubleshooting purposes
		}
	}

	/**
	 * Handles the checkbox column output.
	 *
	 * @param array $item The current order.
	 */
	public function column_cb( $item ) {
		$order_id = intval( $item['Num_Dossier'] );

		// Add data-numexp for label retrieval — 0 if no shipping number
		$numexp = sprintf( 'data-numexp="%d"', intval( $item['Expedition'] ) );

		?>
		<label class="screen-reader-text" for="cb-select-<?php echo $order_id; ?>">
			<?php
				/* translators: %s: Order ID. */
				printf( __( 'Sélectionner la commande n°%s' ), $order_id );
			?>
		</label>
		<input id="cb-select-<?php echo $order_id; ?>" type="checkbox" name="orders" value="<?php echo $order_id; ?>" <?php echo $numexp; ?>>
		<?php
	}

	/**
	 * Handles an incoming ajax request (called from admin-ajax.php)
	 */
	public function display_refresh() {
		$total_pages = $this->_pagination_args['total_pages'];
		$total_items = $this->_pagination_args['total_items'];

		ob_start();
		$this->display_labels_buttons();
		$labels = ob_get_clean();

		ob_start();
		$this->display_rows_or_placeholder();
		$rows = ob_get_clean();

		ob_start();
		$this->print_column_headers();
		$headers = ob_get_clean();

		ob_start();
		$this->pagination( 'top' );
		$pagination_top = ob_get_clean();

		ob_start();
		$this->pagination( 'bottom' );
		$pagination_bottom = ob_get_clean();

		$response = array( 'rows' => $rows );
		$response['pagination']['top']    = $pagination_top;
		$response['pagination']['bottom'] = $pagination_bottom;
		$response['column_headers']       = $headers;
		$response['buttons']              = $labels;

		if ( isset( $total_items ) ) {
			$response['total_items_i18n'] = sprintf( _n( '1 item', '%s items', $total_items ), number_format_i18n( $total_items ) );
		}

		if ( isset( $total_pages ) ) {
			$response['total_pages']      = $total_pages;
			$response['total_pages_i18n'] = number_format_i18n( $total_pages );
		}

		die( json_encode( $response ) );
	}

	/**
	 * Create a CSV with the filtered orders
	 */
	public function export_csv() {
		header( 'Content-Type: text/csv' );
		header( 'Content-Disposition: attachment; filename=mondialrelay.csv' );
		header( 'Pragma: no-cache' );
		header( 'Expires: 0' );

		/*
		 * Column titles
		 */
		$columns = $this->get_columns();

		// Ignore checkbox
		unset( $columns['cb'] );

		// Output header line
		$fp = fopen( 'php://output', 'wb' );

		// BOM header
		fwrite( $fp, "\xEF\xBB\xBF" );

		// Column titles
		fputcsv( $fp, $columns, ';' );

		/*
		 * List of orders
		 */
		$breaks = [ '<br>', '<br/>', '<br />' ];
		foreach ( $this->items as $line ) {

			// Reorder and optionally process columns
			$output = array();
			foreach ( $columns as $code => $title ) {
				switch( $code ) {

					case 'Adresse':
						// Newlines within a field need to be \n
						$output[ $code ] = str_ireplace( $breaks, "\n", $line[ $code ] );
						break;

					case 'Status':
						$output[ $code ] = wc_get_order_status_name( $line[ $code ] );
						break;

					default:
						$output[ $code ] = $line[ $code ];
				}
			}
			fputcsv( $fp, $output, ';' );
		}
		fclose( $fp );

		die();
	}

}
